;
;    JPEGVIEW.ASM
;
;    This program displays jpeg images. The window can be resized.
;
;    Compile with FASM v1.49 for DOS; Menuet's FASM doesn't work (yet)
;
;
;    Version 0.1    7th March 2004
;                   Mike Hibbett ( very small part! )
;    Version 0.11   7th April 2004
;                   Ville Turjanmaa ( 'set_as_bgr' function )
;
;    The *tiny* jpeg rendering software, referenced in the attached .asm files
;    are as supplied by Octavio. I dont yet know what copyright should be applied.

use32

    macro   align value { rb (value-1) - ($ + value-1) mod value }
   
                org     0x0
   
               db     'MENUET01'              ; 8 byte id
               dd     0x01                    ; header version
               dd     START                   ; start of code
               dd     I_END                   ; size of image
               dd     0x100000                 ; memory for app
               dd     0x100000 - 1024           ; esp
               dd     0x0 , 0x0               ; I_Param , I_Icon
   
   
stack_size=4096 + 1024
   
START:                          ; start of execution
    ; Calculate the 'free' memory available
    ; to the application, and create the malloc block from it
    mov     ecx,100000h-fin-stack_size
    mov     edi,fin
    call    add_mem
    
    ; Get some memory
    mov     ecx,16384
    call    malloc
    mov     [work_area],edi
    
    call    colorprecalc

    call    draw_window

still:
    
    mov  eax,10                 ; wait here for event
    int  0x40
   
    cmp  eax,1                  ; redraw request ?
    je   red
    cmp  eax,2                  ; key in buffer ?
    je   key
    cmp  eax,3                  ; button in buffer ?
    je   button

    jmp  still
   
  red:                          ; REDRAW WINDOW
    call draw_window
    jmp  still
   
key:                          ; KEY
    mov  eax,2                  ; get it, but ignore
    int  0x40
       
button:                       ; BUTTON
    mov  eax,17
    int  0x40
    cmp  ah,1                   ; CLOSE PROGRAM
    jne  noclose

    mov  eax,-1
    int  0x40

noclose:

    cmp     ah, 2               ; enter filename
    jne     no_input
    call    read_string
  no_input:

    cmp     ah, 3               ; enter filename
    jne     no_bgr
    call    set_as_bgr
    jmp     still
  no_bgr:

    jmp     still

pixel dd 0x0

py    dd 0x0
px    dd 0x0

wx    dd 0x0
wy    dd 0x0

rx    dd 0x0

sx    dd 0x0
sy    dd 0x0

set_as_bgr:

    pusha

    mov  eax,9
    mov  ebx,0x100000-1024
    mov  ecx,-1
    int  0x40

    mov  eax,[0x100000-1024+34]
    mov  [wx],eax
    mov  eax,[0x100000-1024+38]
    mov  [wy],eax

    mov  eax,61
    mov  ebx,1
    int  0x40

    shr  eax,16
    mov  [rx],eax

    mov  [sx],0
    mov  [sy],0

    mov  esi,[wy]
    add  esi,20
    imul esi,[rx]
    add  esi,[wx]
    add  esi,5

    mov  edi,0

  newyget:

    pusha

  newxget:

    mov  eax,35
    mov  ebx,esi
    int  0x40

    mov  [pixel],eax

    mov  eax,15
    mov  ebx,2
    mov  ecx,edi
    mov  edx,[pixel]
    and  edx,0xffffff
    int  0x40

    add  esi,1
    add  edi,3

    inc  [sx]
    mov  eax,[sx]
    cmp  eax,[px]
    jb   newxget

    popa

    mov  [sx],0

    add  esi,[rx]

    add  edi,[px]
    add  edi,[px]
    add  edi,[px]

    inc  [sy]
    mov  eax,[sy]
    cmp  eax,[py]
    jb   newyget

    mov  eax,15
    mov  ebx,1
    mov  ecx,[px]
    mov  edx,[py]
    int  0x40

    mov  eax,15
    mov  ebx,4
    mov  ecx,2
    int  0x40

    mov  eax,15
    mov  ebx,3
    int  0x40

    popa

    ret

   
    ; Put a 'chunk' of the image on the window
put_image:
    pushad

    push   edx
    	    
    add     dx, ax
    add     dx, 5+2
    cmp     dx, [winxs]
    jnc     .l1

    pop     edx 

    push    ecx
    add     cx, bx
    add     cx, 20+2 + 17
    cmp     cx, [winys]
    jnc     .l1

    pop    ecx

    pusha  ; save for background
    and    eax,0xffff
    and    ebx,0xffff
    and    ecx,0xffff
    and    edx,0xffff
    add    eax,ecx
    add    ebx,edx
    mov    [px],eax
    mov    [py],ebx
    popa

    add     ax,5  ; offset for boarder
    add     bx,20 ; offset for title bar
    push    ax ; pox
    push    bx ; pos
    push    cx ; size
    push    dx ; size
    pop     ecx
    pop     edx
    mov     ebx,edi
    mov     eax,7

    int     40h                         ; Put image function

    popad
    ret
 
.l1:
    pop     ecx
    popad
    ret
   
   
;   *********************************************
;   *******  WINDOW DEFINITIONS AND DRAW ********
;   *********************************************
   
   
draw_window:

    mov  eax,12
    mov  ebx,1
    int  0x40
   
    ; Draw the window to the appropriate size - it may have
    ; been resized by the user
    mov     eax, 0
    cmp     [winxs], ax
    jne     dw_001
    
    ; Give the screen some inital defaults
    mov     ax, 400
    mov     [winxs], ax
    mov     ax, 300
    mov     [winys], ax
    mov     ax, 100
    mov     [winxo], ax
    mov     [winyo], ax
    jmp     dw_002
    
dw_001:    
    mov     eax, 9
    mov ebx, 0x100000 - 1024
    mov ecx, -1
    int     0x40
    mov     eax, [ebx + 34]
    mov     [winxo], ax
    mov     eax, [ebx + 38]
    mov     [winyo], ax
    mov     eax, [ebx + 42]
    mov     [winxs], ax
    mov     eax, [ebx + 46]
    mov     [winys], ax
    
dw_002:
    mov     bx, [winxo]
    shl     ebx, 16
    mov     bx, [winxs]

    mov     cx, [winyo]
    shl     ecx, 16
    mov     cx, [winys]


    mov  eax,0                     ; DRAW WINDOW
    mov  edx,[wcolor]
    add  edx,0x02000000
    mov  esi,0x80557799
    mov  edi,0x00557799
    int  0x40
   
    mov  eax,4                     ; WINDOW LABEL
    mov  ebx,8*65536+8
    mov  ecx,0x00ffffff
    mov  edx,labelt
    mov  esi,labellen-labelt
    int  0x40

   
    mov  eax,8                     ; CLOSE BUTTON                                              
    mov  bx, [winxs]
    sub  bx, 19
    shl  ebx, 16
    add  ebx, 12

    mov  ecx,5*65536+12
    mov  edx,1
    mov  esi,0x557799
    int  0x40

    ; draw status bar
    mov     eax, 13
    movzx     ebx, word [winxs]
    sub     ebx, 5
    add     ebx, 4*65536
    mov     cx, [winys]
    sub     ecx, 19
    shl     ecx, 16
    add     ecx, 3
    mov     edx, 0x00557799
    int     0x40
   
    mov  eax,8                     ; BUTTON 2: filename
    mov  ebx,4*65536+55
    mov  cx, [winys]
    sub  cx, 16
    shl  ecx, 16
    add  ecx, 12
    mov  esi, 0x00557799
    mov  edx,2
    int  0x40

    mov  eax,4                     ; Button text
    movzx ebx, word [winys]
    sub   ebx, 13
    add   ebx, 6*65536
    mov  ecx,0x00ffffff
    mov  edx,setname
    mov  esi,setnamelen-setname
    int  0x40


    mov  eax,8                     ; BUTTON 3: set as background
    mov  bx, [winxs]
    sub  bx, 60
    shl  ebx, 16
    mov  bx,55
    mov  cx, [winys]
    sub  cx, 16
    shl  ecx, 16
    add  ecx, 12
    mov  esi, 0x00557799
    mov  edx,3
    int  0x40

    mov  eax,4                     ; Button text
    movzx ebx, word [winxs]
    sub   ebx, 60
    shl   ebx,16
    mov   bx, word [winys]
    sub   bx,13
    mov  ecx,0x00ffffff
    mov  edx,setbgr
    mov  esi,setbgrlen-setbgr
    int  0x40

    call    load_image
    mov     eax,12                    ; function 12:tell os about windowdraw
    mov     ebx,2                     ; 2, end of draw
    int     0x40

    ret
   


    ; Read in the image file name. 
read_string:

    mov     edi,name_string
    call    print_strings

    mov     edi,name_string-1
    
f10:
    inc     edi
    cmp     byte[edi],0
    jne     f10
    
f11:
    mov     eax,10                     ; Wait for event
    int     0x40
    cmp     eax,2                      ; Key pressed == 2
    jz      fbu
    jmp     rs_done
    
fbu:
    mov     eax,2
    int     0x40                        ; Get the key value
    shr     eax,8
    cmp     eax,13                      ; Return key ends input
    je      rs_done
    cmp     eax,8
    jnz     nobsl
    cmp     edi,name_string
    jz      f11
    dec     edi
    mov     [edi],byte 0;'_'
    call    print_strings
    jmp     f11
nobsl:
    cmp     eax,31
    jbe     f11
    cmp     eax,97
    jb      keyok
    sub     eax,32
    
keyok:
    mov     [edi],al
    call    print_strings

    add     edi,1
    mov     esi,name_string
    add     esi,45
    cmp     esi,edi
    jnz     f11

rs_done:
    mov     al,0
    mov     ebx,name_string+90

newz:
    stosb
    cmp     edi,ebx
    jb      newz
    call    print_strings
    
load_image:

    mov     eax,13              ; clear picture area
    movzx    ebx, word [winxs]
    sub     ebx, 7
    add     ebx, 4 * 65536
    movzx    ecx, word [winys]
    sub     ecx, 39
    add     ecx, 20 * 65536
    
    mov     edx,0
    int     0x40


    mov     esi,name_string
    call    open
    test    eax,eax
    jz      .end2
    call    jpeg_info
    test    ebp,ebp
    jz      .end
    mov     dword [ebp+draw_ptr],put_image
    call    jpeg_display
    call    jpeg_close
    jmp     .end
    
.end2:
    call    close

.end:
    mov     ecx,20000h-fin-stack_size
    mov     edi,fin
    call    add_mem
    
    ; Get some memory
    mov     ecx,16384
    call    malloc
    mov     [work_area],edi

    ret


print_strings:
    pusha
    mov     eax,13              ; clear text area
    movzx   ebx, word [winxs]
    sub     ebx, 64+58
    add     ebx, 60*65536
    mov     cx, [winys]
    sub     cx, 16
    shl     ecx, 16
    add     ecx, 12
    mov     edx,0xffffff
    int     0x40

    mov     eax,4               ;
    movzx   ebx, word [winys]
    sub     ebx, 14
    add     ebx, 60*65536
    mov     ecx,0x000000
    mov     edx,name_string
    mov     esi,60
    int     0x40
    popa
    ret

   
   
include 'filelib.asm'
include 'memlib.asm'
include 'jpeglib.asm'
   
   
; DATA AREA

wcolor          dd  0x000000
labelt          db  'Jpegview v0.1'
labellen:
setname          db  'Filename'
setnamelen:

setbgr           db  '   BGR  '
setbgrlen:


winxo:          dw  0
winyo:          dw  0
winxs:          dw  0
winys:          dw  0
jpeg_st:        dd  0
work_area:      dd  0
tcolor          dd  0x000000
btcolor         dd  0x224466+0x808080
name_string:    db '/rd/1/jpegview.jpg',0

rb 100

align 4

rgb16:          db 0,4,8,13,17,21,25,29,34,38,42,46,50,55,59,63
rgb4:           db 0,21,42,63

include 'jpegdat.asm'

align 4

iniciomemoria:
              dd -(iniciomemoria+4),-(iniciomemoria+4),(iniciomemoria+4),.l1,0
.l1           dd 0

fin:
I_END:
   
   
   
   
